# Barrier python module

import os
import logging
import sys

from functools import wraps

try:
    import unittest2 as unittest # type: ignore
except ImportError:
    import unittest # type: ignore


class TestCase(unittest.TestCase):
    """Wrapper on the unittest TestCase class.

    This class provides setUp and tearDown methods for pySMT in which
    a fresh environment is provided for each test.
    """

#    def setUp(self):
#        self.env = reset_env()
#        self.env.enable_infix_notation = True

    def get_from_path(self, path, must_exists=True):
        current_path = os.path.dirname(os.path.abspath(__file__))
        input_path = os.path.join(current_path, path)
        self.assertTrue((not must_exists) or os.path.exists(input_path))
        return input_path

    def log_to_stdout(self):
        root = logging.getLogger()
        root.setLevel(logging.DEBUG)
        handler = logging.StreamHandler(sys.stdout)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        root.addHandler(handler)

class skipIfGraphvizIsNotAvailable(object):
    """Skip a test if graphviz is not installed."""

    def __init__(self):
        pass

    def __call__(self, test_fun):
        msg = "Graphviz not available"

        try:
            import graphviz
            cond = False
        except ImportError:
            cond = True
        finally:
            @unittest.skipIf(cond, msg)
            @wraps(test_fun)
            def wrapper(*args, **kwargs):
                return test_fun(*args, **kwargs)
            return wrapper

# class skipIfMSaticIsNotAvailable(object):
#     """Skip a test if the given solver is not available."""

#     def __init__(self):
#         pass

#     def __call__(self, test_fun):
#         msg = "MSatic3 not available"
#         cond = MSatic3.find_msatic() is None
#         @unittest.skipIf(cond, msg)
#         @wraps(test_fun)
#         def wrapper(*args, **kwargs):
#             return test_fun(*args, **kwargs)
#         return wrapper

