import logging
import os
import sys


from . import TestCase

from pysmt.shortcuts import *
from mc308.bmc import *

class TestBMC(TestCase):
    def get_vars(self):
        return [Symbol(chr(i))  for i in range(ord('a'), ord('z') + 1)]

    def test_gp_no_loop(self):
        V = self.get_vars()[:3]
        a,b,c = V
        a_next,b_next,c_next = get_nexts(V)

        # TS = ({a,b,c}, !a & !b & !c, True)
        # p = G (a & ! b & c)
        ts = SymbolicTS(V,
                        And([Not(a),Not(b),Not(c)]),
                        TRUE())
        self.assertFalse(bmc_Gp_no_loop(ts, And([a,Not(b),c]), 0))
        self.assertFalse(bmc_Gp_no_loop(ts, And([a,Not(b),c]), 1))

        # TS = ({a,b,c}, !a & !b & !c, a' <-> a)
        # p = G ! a
        #
        ts = SymbolicTS(V,
                        And([Not(a),Not(b),Not(c)]),
                        Iff(a_next,a))
        self.assertTrue(bmc_Gp_no_loop(ts, Not(a), 0))
        self.assertTrue(bmc_Gp_no_loop(ts, Not(a), 1))
        self.assertTrue(bmc_Gp_no_loop(ts, Not(a), 2))

        # TS = ({a,b}, !a & !b, a' <-> a & b' <-> !b)
        # p = G ! a
        #
        ts = SymbolicTS(V,
                        And([Not(a),Not(b)]),
                        And([Iff(a_next,a),
                             Iff(b_next,Not(b))
                             ]
                            )
                        )
        self.assertTrue(bmc_Gp_no_loop(ts, Not(b), 0))
        self.assertFalse(bmc_Gp_no_loop(ts, Not(b), 1))

        # TS = ({a,b}, !a & !b, a' <-> !a & b' <-> (a xor b))
        # p = G ! a
        #
        ts = SymbolicTS(V,
                        And([Not(a),Not(b)]),
                        And([Iff(a_next,Not(a)),
                             Iff(b_next,Xor(a,b))
                             ]
                            )
                        )
        self.assertTrue(bmc_Gp_no_loop(ts, Not(And(a,b)), 0))
        self.assertTrue(bmc_Gp_no_loop(ts, Not(And(a,b)), 1))
        self.assertTrue(bmc_Gp_no_loop(ts, Not(And(a,b)), 2))
        self.assertFalse(bmc_Gp_no_loop(ts, Not(And(a,b)), 3))


    def test_gp(self):
        V = self.get_vars()[:3]
        a,b,c = V
        a_next,b_next,c_next = get_nexts(V)

        # TS = ({a,b}, !a & !b, a' <-> !a & b' <-> (a xor b))
        # p = G ! a
        #
        ts = SymbolicTS(V,
                        And([Not(a),Not(b)]),
                        And([Iff(a_next,Not(a)),
                             Iff(b_next,Xor(a,b))
                             ]
                            )
                        )
        self.assertTrue(bmc_Gp(ts, Not(And(a,b)), 0))
        self.assertTrue(bmc_Gp(ts, Not(And(a,b)), 1))
        self.assertTrue(bmc_Gp(ts, Not(And(a,b)), 2))
        self.assertTrue(bmc_Gp(ts, Not(And(a,b)), 3))
        self.assertFalse(bmc_Gp(ts, Not(And(a,b)), 4))


    def test_fp(self):
        V = self.get_vars()[:2]
        a,b = V
        a_next,b_next = get_nexts(V)

        # TS = ({a,b}, ! a & !b,
        #       (a' <-> a) & (b' <-> b)
        #      )
        # p = F (a & b)
        #
        ts = SymbolicTS(V,
                        And(Not(a),Not(b)),
                        And(
                            Iff(a_next, Not(a)),
                            Iff(b_next,b))
                        )
        P = And(a,b)
        self.assertTrue(bmc_Fp(ts, P, 0))
        self.assertTrue(bmc_Fp(ts, P, 1))
        self.assertFalse(bmc_Fp(ts, P, 2))


    def test_fgp(self):
        V = self.get_vars()[:2]
        a,b = V
        a_next,b_next = get_nexts(V)

        # TS = ({a,b}, a <-> b,
        #       (!b -> ( (a' <-> !a) & !b') &
        #       ( b -> ( !a' & !b') )
        #      )
        # p = FG (a & b)
        #
        ts = SymbolicTS(V,
                        Iff(a,b),
                        And(
                            Implies(Not(b),
                                    And( Iff(a_next, Not(a)),
                                         Not(b_next)
                                       )
                                    ),
                            Implies(b, And(Not(a_next),Not(b_next)) )
                        )
                        )

        P = And(a,b)
        self.assertTrue(bmc_FGp(ts, P, 0))
        self.assertTrue(bmc_FGp(ts, P, 1))
        self.assertFalse(bmc_FGp(ts, P, 2))
